function fractal(action)

% Fraktalgenerator (Mandelbrot- und Juliamengen)
% 
% Author:  Klaus Lichtenegger
% Version: 1.0
% Date:    December 10, 2001
% Status:  works quite well
% 
% User Interface basiert auf Game of Life von W. Kernbichler

% ==========================================================================
func_name = 'fractal';                      % function name - same as declaration
% ==========================================================================
handle_name = [func_name,'_HANDLE'];        % name of the global variable
eval(['global ',handle_name]);              % set it global
if nargin == 0,  action = 'initgraph'; end  % set to initgraph without input
if ~strcmp(action,'initgraph'), 
  eval(['HANDLE=',handle_name,';']);        % copy global to local
else

  % general outfit:
  logo_switch = 1;                  % logo: yes (1), no (0)
  fig_title   = 'Fractals';
  logo_title  = 'Programm zur Erstellung von Mandelbrot- und Juliamengen';
  logo_name   = 'Klaus Lichtenegger, Winfried Kernbichler';
  logo_lv     = 'Multimediale Lehre';
  logo_type   = 'logo_1';           % only this is defined at the moment
  
  % some special values:
  min_dim     = 5;                  % minimum dimension (x- and y-dir)
  max_dim     = 150;                % maximum dimension (x- and y-dir)
  sta_dim     = 40;                 % standard dimension (x- and y-dir)
  min_iter    = 1;                  % minimum number of iterations
  max_iter    = 200;                % maximum number of iterations
  sta_iter    = 100;                % standard number of iterations
  
  % some colors
  ax_color = [0.9,0.9,0.9];         % default axes color
  bo_color = [0.6,0.6,0.6];         % default border color
  
  % do not change
  HANDLE = [];                      % empty at the beginning

end

% ==========================================================================
% ==========================================================================
% Here every aspect of the graphics part should be handled
% This contains the definition and the callback routines
switch lower(action)
  
  % ========================================================================
  % initialization of program
  case 'initgraph'    
    % ----------------------------------------------------------------------
    % preprocessing for initgraph (keep this lines) 
    eval( [func_name,' ''initgraph_pre'''] );
    % ----------------------------------------------------------------------
    
    % ---------------------------------------------------------------------- 
    % definition and creation of graphics objects
    % ----------------------------------------------------------------------

    % ----------------------------------------------------------------------  
    % definition of main figure
    % ----------------------------------------------------------------------  
    F.figure = struct( ...
	'frame',    { [0.0, 0.0, 1.0, 1.0] }, ...
	'geometry', { [1] }, ...
	'closerequestfcn', { 'close_request_main' }, ...
	'renderer', { 'opengl' }, ...
	'name',     { fig_title }, ...
	'tag',      { 'main' } ...
	);
    % --- should be kept ---------------------------------------------------  
    [HANDLE,fig_han] = gui_create(gui_define(F),HANDLE);  clear F;
    % add function name to tag and closerequestfcn
    % important for finding program related figures and to find function
    gui_add_name(fig_han,func_name,'_','b','tag');
    gui_add_name(fig_han,func_name,' ','b','closerequestfcn');
    % sets necessary userdata for figures
    gui_set_userdata(fig_han, 'function',func_name, 'end','exit');
    figmain_han = fig_han;
    % figure color
    fig_color = get(gui_handle_query(HANDLE.figure,'tag','*main*'),'color');
    % ----------------------------------------------------------------------
    
    % ----------------------------------------------------------------------  
    % create meta frames
    % ----------------------------------------------------------------------  
    if logo_switch
      logo_y_pos  = 0.9; % y division between logo and rest
    else
      logo_y_pos  = 1.0;
    end % if logo_switch
    meta_frame_logo   = [0, logo_y_pos, 1, 1-logo_y_pos]; % logo
    meta_frame_layer1 = [0, 0,          1, logo_y_pos  ]; % layer 1   
    % ----------------------------------------------------------------------  

    % ----------------------------------------------------------------------  
    % create logo
    % ----------------------------------------------------------------------
    if logo_switch
      % --------------------------------------------------------------------  
      logo_string = {logo_title,logo_name,logo_lv};
      logo_parent = figmain_han;
      % here one could use HANDLE instead of HANDLE_LOGO, then one could 
      % refere to all contents within the rest of the program.
      % When using HANDLE_LOGO this graphics part stays seperate. 
      HANDLE_LOGO = [];
      HANDLE_LOGO = gui_create_logo(HANDLE_LOGO, logo_string, ...
	  logo_parent, meta_frame_logo, logo_type);
    end % if logo_switch
    % ----------------------------------------------------------------------  
    
    % ---------------------------------------------------------------------- 
    % create main frames at 5 positions
    %  four joined parts with division lines given at 
    %  x: x_div_pos y: y_div_pos
    % ----------------------------------------------------------------------  
    mf = meta_frame_layer1;    % choose meta frame
    x_div_pos = 0.70 * mf(3);  % division line in x direction
    y_div_pos = 0.20 * mf(4);  % division line in y direction
    y_div_ps2 = 0.50 * mf(4);  % 2nd y-division line (right hand side only)
   
    % frames for lower-left, upper-left, lower-right, mid-right, upper-right
    ll_frame = [ mf(1),     mf(2),     x_div_pos,       y_div_pos       ];
    ul_frame = [ mf(1),     y_div_pos, x_div_pos,       mf(4)-y_div_pos ];
    lr_frame = [ x_div_pos, mf(2),     mf(3)-x_div_pos, y_div_pos       ];
    mr_frame = [ x_div_pos, y_div_pos, mf(3)-x_div_pos, y_div_ps2-y_div_pos];
    ur_frame = [ x_div_pos, y_div_ps2, mf(3)-x_div_pos, mf(4)-y_div_ps2 ];
    
    % ----------------------------------------------------------------------  
    % definition of some basis axes system for the background of all five
    % parts defined in the meta frame.
    Fbasis.axes = struct( ...
	'geometry', { [1] }, ...
	'xborder',  { [0.0, 0.0] }, ...
	'yborder',  { [0.0, 0.0] }, ...
	'xgap',     { [0.0] }, ...
	'ygap',     { [0.0] }, ...	
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'on' }, ...
	'xtick',    { [] }, ...
	'ytick',    { [] }, ...
	'color',    { ax_color }, ...
	'xcolor',   { bo_color }, ...
	'ycolor',   { bo_color } );
    
    
    % ======================================================================  
    % lower left position : sliders
    % ---------------------------------------------------------------------- 
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ll_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_left',' ','b','tag');            
    
    % ----------------------------------------------------------------------
    % definitions of slider values
    geom = [ 1, 1, 1, 1 ; 1, 1, 1, 1 ];
    % set important values
    visible = { 'on', 'on', 'on', 'on'; 'on', 'on',  'on', 'on' };
    minval  = { -2, -2, -2,    1;  -2, -2, -2,  10 }; % minimum
    maxval  = {  2,  2,  2,  500;   2,  2,  2, 800 }; % maximum
    svalue  = { .5, -2,  2,   20;  .5, -2,  2, 200 }; % value for slider
    evalue = svalue;                                  % value for edit (string)
    for k = 1:length(svalue(:)), evalue{k} = num2str(svalue{k}); end

    % set strings = variable names
    cab = 'set_value';
    str = { 'Realteil', 'x_min', 'x_max', 'Iterationen'; ...
            'Imagteil', 'y_min', 'y_max', 'Aufloesung' };
    tag = lower(str);
    
    % create variables from string
    var = transpose(flipud(lower(str)));
    
    % callback action to be performed
    cb_ac = 'setup_imagemat'; cb_no = '';
    cbaction = { cb_no, cb_no, cb_no, cb_no; cb_no, cb_no, cb_no, cb_no };
    cbaction = transpose(flipud(lower(cbaction)));
    % what should be changed in the slider
    cb_ch = { 'value','round(##)' }; cb_no = {};
    cbchange_sl = { cb_no, cb_no, cb_no, cb_no; cb_no, cb_no, cb_no, cb_no }; 
    cbchange_sl = transpose(flipud(cbchange_sl));
    % what should be changed in the edit field
    cb_ch = { 'string','num2str((round(str2num(##))))' };	cb_no = {};
    cbchange_ed = { cb_no, cb_no, cb_no, cb_no; cb_no, cb_no, cb_no, cb_no };
    cbchange_ed = transpose(flipud(cbchange_ed));
    % what should be updated when slider changes, see also updatehandle
    update_sl = { { 'value','string','num2str(##)' } };
    update_sl = transpose(flipud(update_sl));
    % what should be updated when edit changes, see also updatehandle    
    update_ed = { { 'string','value','str2num(##)' } };
    update_ed = transpose(flipud(update_ed));

    Fu.uicontrol = struct( ...
	'frame',         { ll_frame }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.3] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { 'slider' }, ...
	'min',           { minval }, ...
	'max',           { maxval }, ...
	'value',         { svalue }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 1 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);
    [HANDLE,slider_han] = gui_create(gui_define(Fu),HANDLE);
    
    % sets variable name in userdata + action and update information
    gui_set_userdata(slider_han,'variable',var); 
    gui_set_userdata(slider_han,'cbaction',cbaction);
    gui_set_userdata(slider_han,'cbchange',cbchange_sl);
    gui_set_userdata(slider_han,'updateproperty',update_sl); 
    gui_add_name(slider_han,'slider',' ','e','tag');    
    
    % add edit field shifted above (look for 'yshift')
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'left');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.065);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'left');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              evalue);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.60);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            'edit_check');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);
    
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_han,'variable',var); 
    gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_set_userdata(edit_han,'cbchange',cbchange_ed);
    gui_set_userdata(edit_han,'updateproperty',update_ed);
    gui_set_userdata(edit_han,'callback',[func_name,' ',cab]);
    gui_add_name(edit_han,'slider_edit',' ','e','tag');    

    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'right');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.12);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'right');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_han,'slider_text',' ','e','tag');    

    % updatehandle for slider (handles of edit)
    edit_cell = mat2cell(edit_han,1,ones(size(edit_han)));
    gui_set_userdata(slider_han,'updatehandle',edit_cell); 
    % updatehandle for edit (handles of slider)
    slider_cell = mat2cell(slider_han,1,ones(size(slider_han)));
    gui_set_userdata(edit_han,'updatehandle',slider_cell); 
    % adds function name to callback    
    gui_add_name([slider_han,edit_han],func_name,' ','b','callback');
    % adds common name to tag
    gui_add_name([slider_han,edit_han,text_han],'lower_left',' ','b','tag');
    % clears structur Fu    
    clear Fu
    % ======================================================================      
    
    
    % ======================================================================  
    % lower right position : Start, Help, Stop, Exit
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',lr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_right',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % define uicontrol pushbutton
    geom = [1,1 ; 1,1];
    str = reshape({ 'Start','Help', 'Stop', 'Exit'}, size(geom));
    tag = lower(str);
    cab = lower(str);
    
    F.uicontrol = struct( ...
	'frame',    { lr_frame }, ...
	'geometry', { geom }, ...
	'parent',   { [1] }, ...
	'style',    { 'pushbutton' }, ...
	'visible',  { 'on' }, ...
	'fontsize', { 0.3 }, ...
	'string',   { str }, ...
	'callback', { cab }, ...
	'tag',      { tag } ...
	);
    [HANDLE,newh] = gui_create(gui_define(F),HANDLE); clear F;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'lower_right',' ','b','tag');            
    % settings for stop handle
    stop_handle = gui_handle_query(newh,'tag','*stop');
    gui_set_userdata(stop_handle,'stop',0);
    set(stop_handle,'enable','off');
    % ======================================================================  
    
    
    % ======================================================================  
    % upper left position : Main Image Matrix
    % ----------------------------------------------------------------------
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ul_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_left',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % add axes for results
    F.axes = struct( ...
	'frame',    { ul_frame }, ...
	'geometry', { [1] }, ...
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'off' }, ...
	'tag',      { 'imagemat' } ...
	);
    [HANDLE,axes_h] = gui_create(gui_define(F),HANDLE);clear F;

    axes_han   = HANDLE.axes;
    im_ax_han  = gui_handle_query(axes_han,'tag','*imagemat');
    figure_han = get(im_ax_han,'parent');    
    axes(im_ax_han);
    
    % switch to determine if picture has to be refreshed
    % within the next call of setup_imagemat:
    setappdata(im_ax_han, 'Refresh', 1);
    
    surf1_han = surface([],[],[],[]);
    set(surf1_han,'tag','imagemat');
    HANDLE = gui_create(surf1_han,HANDLE);             
    % ======================================================================  

    
    % ======================================================================  
    % upper right position : Small Image Matrix
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ur_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'upper_right',' ','b','tag');             
    
    F.axes = struct( ...
	'frame',    { ur_frame }, ...
	'geometry', { [1] }, ...
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'off' }, ...
	'tag',      { 'smallimage' } ...
	);
    [HANDLE,axes_h] = gui_create(gui_define(F),HANDLE); clear F;
    
    axes_han   = HANDLE.axes;
    im_ax_han  = gui_handle_query(axes_han,'tag','*smallimage');
    figure_han = get(im_ax_han,'parent');
    axes(im_ax_han);
    
    % switch to determine if picture has to be refreshed
    % within the next call of setup_imagemat:
    setappdata(im_ax_han, 'Refresh', 1);
    
    surf2_han = surface([],[],[],[]);
    set(surf2_han,'tag','smallimage');
    HANDLE = gui_create(surf2_han,HANDLE);           
    % ======================================================================  
  
    
    % ======================================================================  
    % middle right position (pushbuttons)
    % ----------------------------------------------------------------------  
    % basic axis for background    
    Fbasis.axes = setfield(Fbasis.axes,'frame',mr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'mid_right',' ','b','tag');             
    
    % pushbuttons
    geom = ones(6,1);
    str = reshape( { 'Mode 1','Zoom In','Zoom Out', ...
          'Pick & Draw', 'Save Picture', 'iteratanzeige'}, size(geom));
    tag = lower(str);
    
    % define callback actions:
    cabmode  = 'change_mode';
    cabzoin  = 'zoom_in';
    cabzout  = 'zoom_out';
    cabpick  = 'pick_draw';
    cabsave  = 'save_pic';
        
    cab  = reshape( ... 
	{ cabmode, cabzoin, cabzout, cabpick, cabsave, []}, size(geom));
    
    Fpush.uicontrol = struct( ...
	'frame',    { mr_frame }, ...
	'geometry', { geom }, ...
	'yborder',  { [0.25,0.05] }, ...
	'ygap',     { [0.02] }, ...
	'parent',   { [1] }, ...
	'style',    { 'pushbutton' }, ...
	'visible',  { 'on' }, ...
	'fontsize', { 0.6 }, ...
	'string',   { str }, ...
	'callback', { cab }, ...
	'tag',      { tag }, ...
	'value',    { 1 } ...
	);
    [HANDLE,newh] = gui_create(gui_define(Fpush),HANDLE); clear Fpush;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'mid_right',' ','b','tag');
    
    uicont_han = HANDLE.uicontrol; 
    iterat_han = gui_handle_query(uicont_han,'tag','*iteratanzeige*');
    set(iterat_han, 'style', 'text');

    % ======================================================================  
    
    
    % ----------------------------------------------------------------------  
    % update handle (keep these lines)
    % ----------------------------------------------------------------------  
    eval( [handle_name,'=HANDLE;'] );
    
    % ----------------------------------------------------------------------  
    % postprocessing for initgraph (keep these lines)
    % ----------------------------------------------------------------------  
    eval( [func_name,' ''initgraph_post'''] );
    % ----------------------------------------------------------------------  
    
    % ----------------------------------------------------------------------  
    % set main figure to visible on (keep these lines)   
    % ----------------------------------------------------------------------  
    fig_han = eval([handle_name,'.figure']);   % get content of figure
    fig_han = gui_handle_query(fig_han,'tag',['*main']);
    set(fig_han,'visible','on');          % set figures to visible  
    
    eval( [func_name,' ''start'''] );    

    % ======================================================================
   
  case 'initgraph_pre'               
    % preprocessing for initgraph (keep these lines)    
    % ======================================================================
    gui_default('set') % default values
    % close figures belonging to this program
    fig_han = get(0,'children');   
    fig_han = gui_handle_query(fig_han, ...
	'type','figure','tag',[func_name,'*']);
    delete(fig_han);
    % ======================================================================
  
  case 'initgraph_post'               
    % postprocessing for initgraph (keep these lines)   
    % ======================================================================
    gui_default('remove'); % reset defaults to MATLAB defaults
    % ======================================================================
  
  case 'exit'
  % finishing the program (keep these lines)
  % ========================================================================
    fig_han = eval([handle_name,'.figure']); % get all figures
    fig_han = fig_han(ishandle(fig_han));    % check if exist as handle
    delete(fig_han);                         % closes all figures
    clear('global',handle_name)              % deletes the global variable
    % ======================================================================
  
  case 'close_request_main'  
    % close request main (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function'); % function name
    exit = gui_get_userdata(gcf,'end');      % action for exit
    msg = { ...
	['Closing this figure exits program ',func,'.']; ...
	'Do you really want to exit?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end
    % ======================================================================
    
  case 'close_request_help'  
    % close request for help (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function');
    exit = gui_get_userdata(gcf,'end');
    msg = { ...
	'Closing this figure is not possible.'; ...
	'Do you want to make it invisible?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end    
    % ======================================================================
        
  case 'help'
    % calling help (keep these lines)
    % ======================================================================
    frame = [0.1, 0.1, 0.8, 0.5];
    [HANDLE,new] = gui_help(HANDLE,frame,func_name);
    if new, eval([handle_name,'=HANDLE;']); end % export to global
    % ======================================================================
  
  case 'edit_check'
    % checks string from edit field (keep these lines)
    % ======================================================================
    str    = get(gcbo,'string');
    oldval = get(gcbo,'value');
    minval = get(gcbo,'min');
    maxval = get(gcbo,'max');
    val = str2num(str);
    % handle all error cases
    if ~isempty(val)
      if val<minval
	msg = { 'Value to small!','Value set to minimum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	set(gcbo,'string',num2str(minval));
      elseif val>maxval
	msg = { 'Value to large!','Value set to maximum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	set(gcbo,'string',num2str(maxval));
      end
    else
      msg = { 'Wrong value!','Value set to old value.' };
      e_han = msgbox(msg,'Edit Box','error','modal');
      if isfield(get(gcbo,'userdata'),'updatehandle')
	updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
	updateproperty = gui_get_userdata(gcbo,'updateproperty');
	oldval = get(updatehandle,updateproperty{2});
      end
      set(gcbo,'string',num2str(oldval));
    end
    
    % do the real callback
    if isfield(get(gcbo,'userdata'),'callback')
      eval( gui_get_userdata(gcbo,'callback') );
    end
    % ======================================================================
    
    
  case 'set_value'
    % get values from caller 
    % ======================================================================
    e_str  = gui_get_userdata(gcbo,'variable');
    cbaction = gui_get_userdata(gcbo,'cbaction');
    figure_han = get(gcbo,'parent');
    func_name = gui_get_userdata(figure_han,'function');
    
    if isfield(get(gcbo,'userdata'),'cbchange')
      cbchange = gui_get_userdata(gcbo,'cbchange');
      if ~isempty(cbchange)
	prop = cbchange{1};
	chfunc = cbchange{2};
	e_str = [ 'get(gcbo,''',prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(gcbo,''',prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    % do the update of another handle
    if isfield(get(gcbo,'userdata'),'updatehandle')
      updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
      updateproperty = gui_get_userdata(gcbo,'updateproperty');      
      if ~isempty(updatehandle)
	from_prop = updateproperty{1};
	to_prop   = updateproperty{2};
	chfunc    = updateproperty{3};
	e_str = [ 'get(gcbo,''',from_prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(updatehandle,''',to_prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    if ~isempty(cbaction)
      eval( [func_name,' ',cbaction] );
    end
  
    
  % ======================================================================    
  % Start: start simulation
  % ======================================================================
 
  case 'start'

    % get axes handles
    axes_han = HANDLE.axes;
    im1_han = gui_handle_query(axes_han,'tag','*imagemat*');
    im2_han = gui_handle_query(axes_han,'tag','*smallimage*');
    
    % set both images to be refreshed
    setappdata(im1_han, 'Refresh', 1);
    setappdata(im2_han, 'Refresh', 1);
    
    % setup image
    eval( [func_name,' ''setup_imagemat'''] );
    
  % ======================================================================    
  % Stop: stop calculation (not implemented yet)
  % ======================================================================
    
  case 'stop'
    stop_han = gui_handle_query(HANDLE.uicontrol,'tag','*stop');    
    gui_set_userdata(stop_han,'stop',1);

    
  % ======================================================================    
  % Setup Image Matrix: draw pictures that have to be refreshed
  % ======================================================================
    
  case 'setup_imagemat'
  
    % get slider handles
    uicontrol_han = HANDLE.uicontrol;
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    
    % get variables from slider values
    variables = gui_get_userdata(data_handle,'variable');
    values    = get(data_handle,'value');
    lv = length(values);
    if lv==1
      eval([variables,'=values;'])
    else
      for k = 1:lv
	value = values{k};
	var   = variables{k};
	eval([var,'=value;'])
      end
    end
    
    % So we've got variables realteil, x_min, x_max, ...  with
    % values determined by corresponding sliders and edit fields

    % get handles of iteration display and mode button
    iterat_han = gui_handle_query(uicontrol_han,'tag','*iteratanzeige*');
    chmode_han = gui_handle_query(uicontrol_han,'tag','*mode 1*');
    modestr = get(chmode_han, 'String');
    modus = str2num(modestr(6));
    
    %disable buttons and sliders
    set(uicontrol_han,'enable','off');
    set(iterat_han,'enable','on');

    % get image handles:
    axes_han = HANDLE.axes;
    im1_han = gui_handle_query(axes_han,'tag','*imagemat*');
    im2_han = gui_handle_query(axes_han,'tag','*smallimage*');

    % get refresh status from application data
    refresh1 = getappdata(im1_han, 'Refresh');
    refresh2 = getappdata(im2_han, 'Refresh');    
    
    % get surface handles:    
    surf_han = HANDLE.surface;
    surf1_han = gui_handle_query(surf_han,'tag','*imagemat*');
    surf2_han = gui_handle_query(surf_han,'tag','*smallimage*');

    if modus < 1.5 % small Mandelbrot- and big Julia set
      
      if refresh2==1 % if small image has to be refreshed:
	% calculate small image matrix
        smallim = man_draw([-2,-2,2,2], [200,200], 20, iterat_han);
      end % if
      
      if refresh1==1 % if main image has to be refreshed:
        % calculate main image:
        imagemat = jul_draw([x_min,y_min,x_max,y_max], ...
	  [aufloesung,aufloesung], realteil+i*imagteil, iterationen, iterat_han);
      end % if
        
    else % big Mandelbrot and small Julia set
      
      if refresh2==1 % if small image has to be refreshed:
        % calculate small image matrix:
        smallim = jul_draw([-2,-2,2,2], [200,200], ...
	          realteil+i*imagteil, 20, iterat_han);
      end %if
      
      if refresh1==1 % if main image has to be refreshed:
        % calculate main image:
        imagemat = man_draw([x_min,y_min,x_max,y_max], ...
	  [aufloesung,aufloesung], iterationen, iterat_han);  
      end %if
      
    end %if
      
    % get dimensions of image
    x_stp = (x_max - x_min)/(aufloesung-1);
    y_stp = (y_max - y_min)/(aufloesung-1);
    x_vec = x_min: x_stp: x_max;
    y_vec = y_min: y_stp: y_max;
    [X, Y] = meshgrid(x_vec,y_vec);
    
    % display main image (if necessary)
    if refresh1==1
      
      % display image in surface:
      set(surf1_han, 'Xdata', X, 'Ydata', Y, 'Zdata', imagemat, 'Cdata', imagemat);
      set(im1_han, 'XLim', [x_min, x_max], 'YLim', [y_min, y_max]);
      shading(im1_han, 'interp'); % interpolated shading (looks better)
      
    end %if
     
    % display small image (if necessary)
    if refresh2==1
      
      % get image dimensions:
      x_stp = 4/199; y_stp = 4/199;
      x_vec = -2: x_stp: 2; y_vec = -2: y_stp: 2;
      [X, Y] = meshgrid(x_vec,y_vec);
      
      % display image in surface:
      set(surf2_han, 'Xdata', X, 'Ydata', Y, 'Zdata', smallim, 'Cdata', smallim);
      shading(im2_han, 'interp'); % interpolated shading (looks better)
      
    end %if

    % disable image refresh
    setappdata(im1_han, 'Refresh', 0);
    setappdata(im2_han, 'Refresh', 0);    
    
    % enable buttons and sliders:   
    set(uicontrol_han,'enable','on');
    
  
  % ======================================================================    
  % Change Mode: switch between modes one and two
  %              one ... small Mandelbrot and big Julia set
  %              two ... the other way round
  % ======================================================================
 
 case 'change_mode'
    
    % get uicontrol handles and determine mode
    uicont_han = HANDLE.uicontrol; 
    chmode_han = gui_handle_query(uicont_han,'tag','*mode 1*');
    modestr = get(chmode_han, 'String');
    modus = str2num(modestr(6));
    
    % Change current mode:
    if modus<1.5 % in mode one ?
      % switch to mode two:
      set(chmode_han, 'String', 'Mode 2')
    else % in mode two
      % switch to mode one:
      set(chmode_han, 'String', 'Mode 1')
    end % if
    
    % get image axes handles:
    axes_han = HANDLE.axes;
    im1_han = gui_handle_query(axes_han,'tag','*imagemat*');
    im2_han = gui_handle_query(axes_han,'tag','*smallimage*');

    % declare both images to be redrawn:
    setappdata(im1_han, 'Refresh', 1);
    setappdata(im2_han, 'Refresh', 1);    
    
    % zoom out (and setup_imagemat):
    eval( [func_name,' ''zoom_out'''] );    

    
  % ============================================================================
  % Zoom In: define view area by clicking
  % ============================================================================
 
  case 'zoom_in'
  
    % get input from mouse (2 clicks)
    [X, Y] = ginput (2);
    
    % sort co-ordinates (x)
    if X(1)>X(2)
      x_min = X(2); x_max = X(1);
    else
      x_min = X(1); x_max = X(2);
    end % if
    
    % sort co-ordinates (y)
    if Y(1)>Y(2)
      y_min = Y(2); y_max = Y(1);
    else
      y_min = Y(1); y_max = Y(2);
    end % if
    
    % get slider handles
    uicontrol_han = HANDLE.uicontrol;
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    
    % get handles associated with x- and y-values
    xmin_han = gui_handle_query(data_handle,'tag','*x_min*');
    xmax_han = gui_handle_query(data_handle,'tag','*x_max*');
    ymin_han = gui_handle_query(data_handle,'tag','*y_min*');
    ymax_han = gui_handle_query(data_handle,'tag','*y_max*');
    
    % set input values as new view area:
    % gui_setval is used in order to update edit fields too
    gui_setval(xmin_han, x_min);
    gui_setval(xmax_han, x_max);
    gui_setval(ymin_han, y_min);
    gui_setval(ymax_han, y_max);
    
    % declare main image to be redrawn:
    axes_han = HANDLE.axes;
    im1_han = gui_handle_query(axes_han,'tag','*imagemat*');
    setappdata(im1_han, 'Refresh', 1);

    % setup image:
    eval( [func_name,' ''setup_imagemat'''] );
  
  
  % ============================================================================
  % Zoom Out: Set Min-Max-Values to -2:2
  % ============================================================================
 
  case 'zoom_out'
  
    % get slider handles:
    uicontrol_han = HANDLE.uicontrol;
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    
    % get handles associated with x- and y-values:
    xmin_han = gui_handle_query(data_handle,'tag','*x_min*');
    xmax_han = gui_handle_query(data_handle,'tag','*x_max*');
    ymin_han = gui_handle_query(data_handle,'tag','*y_min*');
    ymax_han = gui_handle_query(data_handle,'tag','*y_max*');
    
    % set to maximum/minimum value:
    gui_setval(xmin_han, -2);
    gui_setval(xmax_han,  2);
    gui_setval(ymin_han, -2);
    gui_setval(ymax_han,  2);

    % declare main image to be redrawn:    
    axes_han = HANDLE.axes;
    im1_han = gui_handle_query(axes_han,'tag','*imagemat*');
    setappdata(im1_han, 'Refresh', 1);

    % redraw picture:
    eval( [func_name,' ''setup_imagemat'''] );
    

  % ============================================================================
  % Pick and Draw: get parameter with mouse click
  % ============================================================================
  
  case 'pick_draw'
    
    % get point from mouse click
    [re, im] = ginput (1);
     
    % get slider handles:
    uicontrol_han = HANDLE.uicontrol;
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    
    % get parameter values:
    real_han = gui_handle_query(data_handle,'tag','*realteil*');
    imag_han = gui_handle_query(data_handle,'tag','*imagteil*');
    
    % set parameter:
    gui_setval(real_han, re);
    gui_setval(imag_han, im);
    
    % determine mode (one or two):
    chmode_han = gui_handle_query(uicontrol_han,'tag','*mode 1*');
    modestr = get(chmode_han, 'String');
    modus = str2num(modestr(6));
    
    % get axes handles
    axes_han = HANDLE.axes;
    im2_han = gui_handle_query(axes_han,'tag','*smallimage*');
    
    if modus < 1.5 % in mode one
      
      % zoom out (and setup_imagemat):
      eval( [func_name,' ''zoom_out'''] );
    
    else % in mode two
      
      % redraw small image only:
      setappdata(im2_han, 'Refresh', 1);      
      eval( [func_name,' ''setup_imagemat'''] );     
      
    end % if

    
  % ============================================================================
  % Save Picture: write actual picture to disk
  % ============================================================================
    
 case 'save_pic'
  
    % get main surface handle:    
    surf_han = HANDLE.surface;
    surf1_han = gui_handle_query(surf_han,'tag','*imagemat*');

    % get image data from surface:
    imagemat = get(surf1_han, 'Cdata');
    maxent = max(max(imagemat));
    minent = min(min(imagemat));
    imagemat = 63*(imagemat-minent)/(maxent-minent);
    imagemat = uint8(imagemat);
  
    % file to store picture in
    [newfile,newpath] = uiputfile(...
	'*.tif; *.tiff; *.jpg; *.jpeg; *.bmp; *.png; *.hdf; *.pcx;', ...
	'Save file name');
    
    % write image to file
    if newfile~=0
      fname = [newpath,newfile];     
      imwrite(imagemat, jet, fname);
    end % if
    
 
    
  % ========================================================================
    
  otherwise
    % ======================================================================
    % warnings if callback routine is not defined (keep this at the end)    
    % ======================================================================
    disp(['Callback not defined: ',action]);
    % ======================================================================

end










