% picmat = jul_draw(rect, linsize, para, iter, grhandle)
%
% Fhrt Iterationsschritte fr eine Juliamenge durch
% (C) Klaus Lichtenegger, 2001
%
% rect     ... Koordinaten des betrachteten Ausschnitts  rect = [x_min, y_min, x_max, y_max]
% linsize  ... lineare Abmessung des Ausschnitts         linsize = [x_linsize, y_linsize]
% para     ... komplexer Fraktalparameter
% iter     ... Zahl der Iterationsschritte
% grhandle ... Handle-Vektor fr Ausgabe:
%              []       ... keine Ausgabe
%              h_text   ... Zahl der Iterationen

function picmat = jul_draw(rect, linsize, para, iter, grhandle)

% Koordinaten aus dem rect-Vektor auslesen:
x_min = rect(1); y_min = rect(2);
x_max = rect(3); y_max = rect(4);

% Vektoren entsprechender Lnge generieren:
x_stp = (x_max - x_min)/(linsize(1)-1);
y_stp = (y_max - y_min)/(linsize(2)-1);
x_vec = x_min: x_stp: x_max;
y_vec = y_min: y_stp: y_max;

% Komplexe Wertematrix und dazugehrige Bildmatrix erstellen
[x_mat, y_mat] = meshgrid(x_vec, y_vec);
z_mat = x_mat +i*y_mat; pict = z_mat.*0;

% Iterationen durchfhren:
for I=1:iter
   
   % Die Matrix wird gem Vorschrift iteriert:
   z_mat = z_mat.^2 + para;
   
   % Fr alle Elemente, die betragsmig die kritische Schwelle berschreiten,
   % wird der Pixel der Anzeige auf den logarithm. Iterationswert gesetzt:
   pict = pict + ((abs(z_mat)>=10) & (pict == 0))*log(I);
   
   % Wenn Handle bergeben,
   if length(grhandle)>0
      
      % Anzeige der Iterationen
      set(grhandle, 'String', num2str(I));
      
   end %if length(grhandle)>0
      
end %for I

%Rckgabe des fertigen Bildes:
picmat = pict;

